import os
import parsing
import generators
from logic import *
from agent import GeneralizingAgent

def withoutBidirectionalEdge(clause):
    '''
    Returns true if the clause contains two literals among which there is are bidirectional edges, e.g. {edge(1,2) | edge(2,1)}; otherwise false.

    :type clause: Clause
    :rtype: bool
    '''
    strings = set(map(lambda literal: str(literal), clause.literals))
    for literal in filter(lambda literal: literal.atom.predicate.arity > 1,clause.literals):
        oppositeDirection = Literal(Atom(literal.atom.predicate, [literal.atom.terms[1], literal.atom.terms[0]]),
                                    literal.positive)
        if str(oppositeDirection) in strings:
            # throwing away because there is a bidirectional edge
            return False
    return True


def session(pathToData, s, t, showMostSpecificCNF=False, showOnlyDirectedGraphs=True):
    '''
    Runs the session given path to dataset, s and t constrains, and agent class.


    Prints most specific CNF if showMostSpecificCNF is set to True.

    Prints only clauses corresponding to directed graphs if showOnlyDirectedGraphs is se to True.

    :type pathToData: string
    :type s: int,>=0
    :type t: int,>=0
    :type showMostSpecificCNF: bool
    :type showOnlyDirectedGraphs: bool
    :rtype: None
    '''
    parser = parsing.Parser(pathToData)
    dataset = parser.parse()



    cnf = generators.MostSpecificCNFGenerator(dataset.getPredicates(), s, t, constants=dataset.getConstants()).generate()

    if showMostSpecificCNF:
        print('most specific CNF', len(cnf.clauses))
        for c in cnf.clauses:
            print('', c)

    print('init the agent')
    agent = GeneralizingAgent(cnf)

    for sample in dataset.samples:
        print('feeding agent with',sample)
        agent.feedSample(sample)

    print('end of feeding')

    print('agent hypothesis is')

    hypothesis = agent.getCNF()
    if showOnlyDirectedGraphs:
        print('-', 'showing only directed graphs')
        hypothesis = CNF(filter(lambda clause: withoutBidirectionalEdge(clause), hypothesis.clauses),
                         hypothesis.variableFactory)

    print(hypothesis)
    for clause in hypothesis.clauses:
        print(clause)

    print(len(hypothesis.clauses))


if __name__ == "__main__":

    #pathToDataset = os.sep.join([".", "data", "threeChain"]) # negative class si generated from e(X,Y),e(Y,Z),e(Z,W), , s=t=3 is sufficient
    pathToDataset = os.sep.join([".", "data", "lectureGraphs"])  # negative class is generated from triangles, s=t=3 is sufficient
    s = 3  # maximal number of literals in a clause
    t = 3  # maximal number of symbols in a literal
    # myAgent = agent.GeneralizingAgent

    session(pathToDataset, s, t, showMostSpecificCNF=True, showOnlyDirectedGraphs=True)
